/* SPDX-License-Identifier: 0BSD */

/*
 * Author: Lasse Collin
 */

#define MY_TYPE VFT_APP
#define MY_NAME "lzmainfo"
#define MY_SUFFIX ".exe"
#define MY_DESC "lzmainfo shows information about .lzma files"
#include "common_w32res.rc"
